package no.statnett.ecp.brs.actions;

import no.statnett.ecp.brs.Config;
import no.statnett.ecp.brs.Div;
import no.statnett.ecp.brs.state.MessageFilter;
import no.statnett.ecp.brs.state.MessageInfo;
import no.statnett.ecp.brs.state.QueueInfo;
import no.statnett.ecp.utils.ArtemisConsoleAPI;
import no.statnett.ecp.utils.Const;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ListMessages {

    public static class Print {
        public static void printMI(List<MessageInfo> messages, boolean echo) {
            int i = 1;


            if (echo) {
                String hostHd = "Host (1)";
                String queueHd = "Queue (2)";
                String ageHd = "Age(m) (3)";
                String receiverHd = "Receiver (4)";
                String senderHd = "Sender (5)";
                String msgTypeHd = "MessageType (6)";
                String baMsgIdHd = "BaMessageID (7)";
                String intTypeHd = "InternalType (8)";

                int hostWidth = Math.max(messages.stream().mapToInt(m -> m.getHost().length()).max().orElse(0), hostHd.length());
                int queueWidth = Math.max(messages.stream().mapToInt(m -> m.getQueueName().length()).max().orElse(0), queueHd.length());
                int ageWidth = Math.max(messages.stream().mapToInt(m -> ("" + m.getAge()).length()).max().orElse(0), ageHd.length());
                int receiverWidth = Math.max(messages.stream().mapToInt(m -> m.getReceiverCode().length()).max().orElse(0), receiverHd.length());
                int senderWidth = Math.max(messages.stream().mapToInt(m -> m.getSenderCode().length()).max().orElse(0), senderHd.length());
                int msgTypeWidth = Math.max(messages.stream().mapToInt(m -> m.getMessageType().length()).max().orElse(0), msgTypeHd.length());
                int baMsgIdWidth = Math.max(messages.stream().mapToInt(m -> m.getBaMessageID().length()).max().orElse(0), baMsgIdHd.length());
                int intTypeWidth = Math.max(messages.stream().mapToInt(m -> m.getInternalType().length()).max().orElse(0), intTypeHd.length());
                int totalWidth = hostWidth + queueWidth + ageWidth + receiverWidth + senderWidth + msgTypeWidth + baMsgIdWidth + intTypeWidth + 4 + 8 * 3;


                System.out.println(String.format("%-4s | %-" + hostWidth + "s | %-" + queueWidth + "s | %" + ageWidth + "s | %-" + receiverWidth + "s | %-" + senderWidth + "s | %-" + msgTypeWidth + "s | %-" + baMsgIdWidth + "s | %" + intTypeWidth + "s", "#", hostHd, queueHd, ageHd, receiverHd, senderHd, msgTypeHd, baMsgIdHd, intTypeHd) + " (" + LocalDateTime.now().format(Const.localTmsSec) + ")");

                System.out.println("-".repeat(totalWidth));
                for (MessageInfo mi : messages) {
                    String sb = String.format("%-4d |", i) +
                            String.format(" %-" + hostWidth + "s |", mi.getHost()) +
                            String.format(" %-" + queueWidth + "s |", mi.getQueueName()) +
                            String.format(" %" + ageWidth + "s |", mi.getAge()) +
                            String.format(" %-" + receiverWidth + "s |", mi.getReceiverCode()) +
                            String.format(" %-" + senderWidth + "s |", mi.getSenderCode()) +
                            String.format(" %-" + msgTypeWidth + "s |", mi.getMessageType()) +
                            String.format(" %-" + baMsgIdWidth + "s |", mi.getBaMessageID()) +
                            String.format(" %-" + intTypeWidth + "s", mi.getInternalType());
                    System.out.println(sb);
                    i++;
                }
            }
        }
    }

    public static class Retrieve {
        public static List<MessageInfo> retrieveAndFilter(List<QueueInfo> qiList, Config config, MessageFilter messageFilter) {
            List<MessageInfo> MIList = new ArrayList<>();
            Map<String, String> broker2Encoded = new HashMap<>();
            for (QueueInfo qi : qiList) {
                try {
                    if (broker2Encoded.get(qi.getHost()) == null) { // The same broker on the same host, no need to look up again
                        String brokerNameEncoded = ArtemisConsoleAPI.retrieveBrokerNameURLEncoded(Div.prot(config, qi), qi.getHost(), config.get("port"), Div.user(config, qi), Div.pass(config, qi));
                        broker2Encoded.put(qi.getHost(), brokerNameEncoded);
                    }
                    ArtemisConsoleAPI.ACAPIResult result = ArtemisConsoleAPI.listMessages(Div.prot(config, qi), qi.getHost(), config.get("port"), broker2Encoded.get(qi.getHost()), Div.user(config, qi), Div.pass(config, qi), qi.getQueueName(), messageFilter.getFilterStr());
                    if (result.getStatus().equals("200")) {
                        MIList.addAll(retrieve(qi, result.getValue()));
                    } else {
                        System.out.println("Failed to run listMessages-API for queue: " + qi.getQueueName() + " on host: " + qi.getHost() + " - " + result.getJson());
                    }
                } catch (Exception e) {
                    System.out.println("Error occurred: During listMessages-API for queue: " + qi.getQueueName() + " on host " + qi.getHost() + ": " + e + " " + e.getMessage());
                }
            }

            return MIList;
        }

        private static List<MessageInfo> retrieve(QueueInfo qi, String dataArrJson) {
            dataArrJson = dataArrJson.replaceAll("\\\\", "");
            List<MessageInfo> mInfoList = new ArrayList<>();
            if (dataArrJson.trim().length() < 10) {
                return mInfoList;
            }
            for (String elementJson : dataArrJson.split("},\\{")) {
                // Each element looks like this (except first is prefixed with "{" and last is postfixed with "}".
                // "x-opt-jms-dest":0,"creationTime":1721641647472,"x-opt-jms-msg-type":4,"subject":"HEARTBEAT","messageID":161930336,"CamelJmsDeliveryMode":2,"receiverCode":"50V-EIDSIVAN-ATI","messageMversion":2,"userID":"ID:ID:2f070daf-ecd0-4177-b4f2-2e0f1c1dad3c:4:56126:1-1","durable":true,"messageType":"HEARTBEAT","generated":1721641647221,"x-opt-ORIG-QUEUE":"ecp.endpoint.receive.50V-EIDSIVAN-ATI","baMessageID":"","_AMQ_ACTUAL_EXPIRY":1721641707729,"_AMQ_ORIG_ADDRESS":"ecp.endpoint.receive.50V-EIDSIVAN-ATI","timestamp":1721641647472,"_AMQ_AD":"ExpiryQueue","x-opt-ACTUAL-EXPIRY":1721641707729,"address":"ExpiryQueue","_AMQ_ORIG_ROUTING_TYPE":1,"internalType":"TRACING_MESSAGE","senderCode":"45V000000000059N","x-opt-ORIG-ROUTING-TYPE":1,"priority":4,"x-opt-ORIG-ADDRESS":"ecp.endpoint.receive.50V-EIDSIVAN-ATI","_AMQ_ORIG_MESSAGE_ID":161920626,"x-opt-ORIG-MESSAGE-ID":161920626,"_AMQ_ORIG_QUEUE":"ecp.endpoint.receive.50V-EIDSIVAN-ATI","expiration":0,"senderApplication":"","to":"ExpiryQueue"
                // All attributes we care about are on this form:
                // "key":"value",
                // The values we're interested in, and how they relate to QueueInfo: baMessageID, generated, internalType, messageType, receiverCode, senderApplication, senderCode
                MessageInfo mInfo = new MessageInfo();
                mInfo.setHost(qi.getTrimmedHost());
                mInfo.setQueueName(qi.getQueueName());
                mInfo.setBaMessageID(Div.findStringInElement(elementJson, "baMessageID"));
                String generated = Div.findNumberInElement(elementJson, "generated");
                if (generated.equals("-1"))
                    generated = Div.findNumberInElement(elementJson, "timestamp");
                mInfo.setGenerated(Long.parseLong(generated));
                mInfo.setInternalType(Div.findStringInElement(elementJson, "internalType"));
                String messageType = Div.findStringInElement(elementJson, "messageType");
                if (messageType.equals("")) {
                    messageType = Div.findStringInElement(elementJson, "businessType");
                }
                mInfo.setMessageType(messageType);
                mInfo.setReceiverCode(Div.findStringInElement(elementJson, "receiverCode"));
                mInfo.setSenderCode(Div.findStringInElement(elementJson, "senderCode"));
                mInfo.setMessageID(Div.findNumberInElement(elementJson, "messageID"));
                mInfoList.add(mInfo);
            }
            return mInfoList;
        }
    }


}
