package no.statnett.ecp.brs.state;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class MessageFilter {

  private int minAgeMinutes = 0;
  private String messageType;
  private String baMessageId;
  private String sender;
  private String receiver;

  public int getMinAgeMinutes() {
    return minAgeMinutes;
  }

  public void setMinAgeMinutes(int minAgeMinutes) {
    this.minAgeMinutes = minAgeMinutes;
  }

  public String getMessageType() {
    return messageType;
  }

  public void setMessageType(String messageType) {
    this.messageType = messageType;
  }

  public String getBaMessageId() {
    return baMessageId;
  }

  public void setBaMessageId(String baMessageId) {
    this.baMessageId = baMessageId;
  }

  public String getSender() {
    return sender;
  }

  public void setSender(String sender) {
    this.sender = sender;
  }

  public String getReceiver() {
    return receiver;
  }

  public void setReceiver(String receiver) {
    this.receiver = receiver;
  }

  public String getFilterStr() {
    List<String> filters = new ArrayList<>();
    if (minAgeMinutes <= 0) { // age-filter will always be added, because the Artemis API require some filter (not empty string)
      filters.add("%28AMQTimestamp%3E0%29");
    } else {
      long minAgeMillis = minAgeMinutes * 60 * 1000L;
      long now = System.currentTimeMillis();
      long minAge = now - minAgeMillis;
      filters.add("%28AMQTimestamp%3C" + minAge + "%29");
    }
    if (messageType != null)
      filters.add(createFilterExpression(messageType, "messageType", "businessType", "EDX_MESSAGE_TYPE"));
    if (baMessageId != null)
      filters.add(createFilterExpression(baMessageId, "baMessageId"));
    if (sender != null)
      filters.add(createFilterExpression(sender, "sender", "senderCode", "EDX_SENDER"));
    if (receiver != null)
      filters.add(createFilterExpression(receiver, "receiver", "receiverCode", "EDX_RECEIVER_CODE"));

    return String.join("%20AND%20", filters);

  }

  private String createFilterExpression(String filter, String... propertyNames) {
    String equal = "%3D";
    String unequal = "%3C%3E";
    String like = "%20LIKE%20";
    String unlike = "%20NOT%20LIKE%20";
    boolean not = filter.startsWith("!");
    filter = not ? filter.substring(1) : filter; // remove the ! if present

    String operator = filter.contains("%") ? (not ? unlike : like) : (not ? unequal : equal);
    String filterEscaped = filter.replace("%", "%25"); // Escape % in the message type
    List<String> elements = new ArrayList<>();
    for (String propertyName : propertyNames) {
      elements.add(propertyName + operator + "%27" + filterEscaped + "%27");
    }
    String filterTranslated = "%28" + String.join("%20OR%20", elements) + "%29";
    return filterTranslated;
  }
}
