package no.statnett.ecp.utils;

import java.io.IOException;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.util.Map;

public class ArtemisConsoleAPI {

    public static void main(String[] args) throws IOException, NoSuchAlgorithmException, KeyManagementException {
        String username = "endpoint";
        String password = "password";
        String protocol = "https";
        String host = "h1-a-ecp-apt02.statnett.no";

        String port = "8161";
//        String queueName = "ecp.endpoint.upload.45V0000000000318";

        System.out.println("Broker-name: " + retrieveBrokerNameURLEncoded(protocol, host, port, username, password));
//        printQueueInfo(host, port, brokerName, username, password);
//        printRemovedMessages(host, port, brokerName, username, password, queueName);
    }


    // Broker-list QueryString /console/jolokia/search/org.apache.activemq.artemis:broker=*"
    public static String retrieveBrokerNameURLEncoded(String protocol, String host, String port, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/search/org.apache.activemq.artemis:broker=*");
        String json = getJSON(url, username, password);
        // Example of json: {"request":{"mbean":"org.apache.activemq.artemis:broker=*","type":"search"},"value":["org.apache.activemq.artemis:broker=\"ECCo SP Artemis\""],"timestamp":1720632774,"status":200}
        // Extract broker-name from json
        int valuesPos = json.indexOf("value\":") + 9;
        int brokerNameStartPos = json.indexOf("broker=", valuesPos) + 7;
        int brokerNameEndPos = json.indexOf("]", brokerNameStartPos);
        String brokerNameWithQuotes = json.substring(brokerNameStartPos, brokerNameEndPos);
        String brokerName = brokerNameWithQuotes.replaceAll("\"", "").replaceAll("\\\\", "");
        return brokerName.replaceAll(" ", "%20");
    }

    // Broker-info QueryString  /console/jolokia/read/org.apache.activemq.artemis:broker=%22ECCo%20SP%20Artemis%22
    public static String retrieveBrokerInfoJSON(String protocol, String host, String port, String brokerNameEncoded, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/read/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22");
        return getJSON(url, username, password);
    }

    // Queue-info QueryString:  /console/jolokia/exec/org.apache.activemq.artemis:broker=%22ECCo%20SP%20Artemis%22/listQueues/%7B%22field%22:%22name%22%2C%22operation%22:%22CONTAINS%22%2C%22value%22:%22ecp%22%7D/1/1000
    public static String retrieveQueueInfoJSON(String protocol, String host, String port, String brokerNameEncoded, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/exec/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22/listQueues/%7B%22field%22:%22name%22%2C%22operation%22:%22NOT%5FCONTAINS%22%2C%22value%22:%22abcdefghijkl%22%7D/1/1000");
        return getJSON(url, username, password);
    }


    // RemoveMessages QueryString: /console/jolokia/exec/org.apache.activemq.artemis:broker=%22ECCo%20SP%20Artemis%22%2Ccomponent=addresses%2Caddress=%22QUEU%22%2Csubcomponent=queues%2Crouting-type=%22anycast%22%2Cqueue=%22QUEUE%22/removeAllMessages
    public static ACAPIResult removeAllMessages(String protocol, String host, String port, String brokerNameEncoded, String username, String password, String queueName) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String jsonResponse = getJSON(new URL(protocol + "://" + host + ":" + port + getQueryStringQueue(brokerNameEncoded, queueName) + "/removeAllMessages"), username, password);
        String status = findAttributeValue(jsonResponse, "status");
        String value = findAttributeValue(jsonResponse, "value");
        return new ACAPIResult(status, value, jsonResponse);
    }

    public static ACAPIResult removeMessages(String protocol, String host, String port, String brokerNameEncoded, String username, String password, String queueName, String filter) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String jsonResponse = getJSON(new URL(protocol + "://" + host + ":" + port + getQueryStringQueue(brokerNameEncoded, queueName) + "/removeMessages%28java.lang.String%29/"+filter), username, password);
        String status = findAttributeValue(jsonResponse, "status");
        String value = findAttributeValue(jsonResponse, "value");
        return new ACAPIResult(status, value, jsonResponse);
    }

    // ListMesageInfo:             /console/jolokia/exec/org.apache.activemq.artemis:broker=%22ECCo%20SP%20Artemis%22%2Ccomponent=addresses%2Caddress=%22QUEUE%22%2Csubcomponent=queues%2Crouting-type=%22anycast%22%2Cqueue=%22QUEUE%22/listMessages"
    public static ACAPIResult listMessages(String protocol, String host, String port, String brokerNameEncoded, String username, String password, String queueName, String filter) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String jsonResponse = getJSON(new URL(protocol + "://" + host + ":" + port + getQueryStringQueue(brokerNameEncoded, queueName) + "/listMessages/"+filter), username, password);
        String status = findAttributeValue(jsonResponse, "status");
        String valueArray = findValueArray(jsonResponse, "value");
        return new ACAPIResult(status, valueArray, jsonResponse);
    }


    private static String getQueryStringQueue(String brokerNameEncoded, String queueName) {
        return "/console/jolokia/exec/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22%2Ccomponent=addresses%2Caddress=%22" + queueName + "%22%2Csubcomponent=queues%2Crouting-type=%22anycast%22%2Cqueue=%22" + queueName + "%22";
    }

    public static boolean destroyQueue(String protocol, String host, String port, String brokerNameEncoded, String username, String password, String queueName) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        // Keep in reserve: %2Ccomponent=addresses%2Caddress=%22" + queueName + "%22%2Csubcomponent=queues%2Crouting-type=%22anycast%22%2Cqueue=%22" + queueName + "%22
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/exec/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22/destroyQueue(java.lang.String)/" + queueName);
        String json = getJSON(url, username, password);
        String status = findAttributeValue(json, "status");
        if (status.equals("200"))
            return true;
        else
            return false;
    }

    private static String getJSON(URL url, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        Map<String, String> requestHeaders = Map.of("Accept", "application/json");
        String str = HTTP.retrieveResponseBody("GET", url, username, password, requestHeaders, null);
        if (str == null) return "";
        if (str.contains("error_type")) {
            // We tried to check the connection.getReponseMessage() != 200, but it did not work as expected, so we resort to checking the output
            String errorMessage = "Error occurred: HTTP-response contains errors: " + str + " for URL: " + url + " using user/pass " + username + "/" + password;
            System.out.println("\n" + errorMessage);
            throw new IOException(errorMessage);
        }
        return str;

    }

    private static String findAttributeValue(String json, String name) {
        int statusStartPos = json.indexOf(name + "\":") + name.length() + 2;
        if (statusStartPos <= (-1 + name.length() + 2))
            return null;
        int statusEndPos = json.indexOf(",", statusStartPos) == -1 ? json.indexOf("}", statusStartPos) : json.indexOf(",", statusStartPos);
        return json.substring(statusStartPos, statusEndPos);
    }

    private static String findValueArray(String json, String name) {
        int statusStartPos = json.indexOf(name + "\":[") + name.length() + 3;
        if (statusStartPos <= (-1 + name.length() + 2))
            return null;
        int statusEndPos = json.indexOf("]", statusStartPos);
        return json.substring(statusStartPos, statusEndPos);
    }


    public static class ACAPIResult {
        private String status;
        private String value;
        private String json;

        public ACAPIResult(String status, String value, String json) {
            this.status = status;
            this.value = value;
            this.json = json;
        }

        public String getStatus() {
            return status;
        }

        public String getValue() {
            return value;
        }

        public String getJson() {
            return json;
        }

        public String toString() {
            return "Status: " + status + ", Value: " + value + ", JSON: " + json;
        }
    }
}
