package no.statnett.ecp.utils;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;

public class EcpHTTP {
  public static String getECPHTTResponse(String method, URL url, String user, String pw, String body) throws NoSuchAlgorithmException, KeyManagementException, IOException {
      Map<String, String> requestHeaders = new HashMap<>(Map.of("Accept", "application/json"));
      URL loginURL = new URL(url.getProtocol(), url.getHost(), url.getPort(), "/ECP_MODULE/");
      HttpURLConnection connection = HTTP.connect("GET", loginURL, user, pw, requestHeaders, null);
      String XSRF = "XSRF-TOKEN";
      String JSESSIONID = "JSESSIONID";
      String xsrfToken = getCookieValue(connection, XSRF);
      String jsessionid = getCookieValue(connection, JSESSIONID);
      requestHeaders.put("Cookie", XSRF + "=" + xsrfToken + "; " + JSESSIONID + "=" + jsessionid + "; role=admin");
      requestHeaders.put("X-Xsrf-Token", xsrfToken);
      requestHeaders.put("Content-Type", "application/json;charset=UTF-8");
      return HTTP.retrieveResponseBody(method, url, user, pw, requestHeaders, body);
  }

  private static String getCookieValue(HttpURLConnection connection, String cookieKey) {
      for (String headerKeys : connection.getHeaderFields().keySet()) {
          int i = 0;
          for (String header : connection.getHeaderFields().get(headerKeys)) {
              if (header != null && header.contains(cookieKey)) {
                  String cookie = header;
                  int cookieStartPos = cookie.indexOf(cookieKey + "=") + cookieKey.length() + 1;
                  int cookieEndPos = cookie.indexOf(";", cookieStartPos);
                  if (cookieEndPos == -1) {
                      return cookie.substring(cookieStartPos);
                  } else {
                      return cookie.substring(cookieStartPos, cookieEndPos);
                  }
              }
          }
      }
      return null;
  }
}
