package no.statnett.ecp.utils;

import javax.net.ssl.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.Base64;
import java.util.Map;

public class HTTP {
    public static String retrieveResponseBody(String method, URL url, String username, String password, Map<String,String> requestHeaders, String requestPayload) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        HttpURLConnection connection = connect(method, url, username, password, requestHeaders, requestPayload);

        try {
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(connection.getResponseCode() == 200 ? connection.getInputStream() : connection.getErrorStream(), StandardCharsets.UTF_8));
            String line;
            StringBuilder builder = new StringBuilder();
            while ((line = bufferedReader.readLine()) != null) {
                builder.append(line+"\n");
            }
            bufferedReader.close();
            connection.disconnect();
            return builder.toString();

        } catch (Exception e) {
            System.out.println("\nError occurred: Connection to URL: " + url + " using user/pass " + username + "/" + password + " failed - we expect Artemis-console to be available on HTTPS on port 8161 (hardcoded). Check user/pass + firewall + jolokia-access.xml to ensure access.");
            throw e;
        }
    }

    // Performs GET with Basic Authentication, handles both HTTP and HTTPS (but disregards certificate validation) -
    public static HttpURLConnection connect(String method, URL url, String username, String password, Map<String,String> requestHeaders, String requestPayload) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        HttpURLConnection connection;
        try {
            if (url.getProtocol().equals("https")) {
                // Step 2: Set up a TrustManager that trusts all certificates
                TrustManager[] trustAll = new TrustManager[]{
                        new X509TrustManager() {
                            @Override
                            public X509Certificate[] getAcceptedIssuers() {
                                return null;
                            }

                            @Override
                            public void checkClientTrusted(X509Certificate[] certs, String authType) {
                            }

                            @Override
                            public void checkServerTrusted(X509Certificate[] certs, String authType) {
                            }
                        }
                };
                SSLContext sslContext = SSLContext.getInstance("TLS");
                sslContext.init(null, trustAll, null);

                HttpsURLConnection.setDefaultSSLSocketFactory(sslContext.getSocketFactory());

                // Step 4: Set up a HostnameVerifier that accepts all hostnames
                HostnameVerifier allHostsValid = (hostname, session) -> true;
                HttpsURLConnection.setDefaultHostnameVerifier(allHostsValid);

                // Step 5: Open a connection
                connection = (HttpsURLConnection) url.openConnection();

            } else {
                // Step 5: Open a connection
                connection = (HttpURLConnection) url.openConnection();

            }
        } catch (Exception e) {
            System.out.println("\nError occurred: Connection to URL: " + url + " using user/pass " + username + "/" + password + " failed - we expect Artemis-console to be available on HTTPS on port 8161 (hardcoded). Check user/pass + firewall + jolokia-access.xml to ensure access.");
            throw e;
        }

        try {
            connection.setRequestMethod(method);
            for (Map.Entry<String, String> entry : requestHeaders.entrySet()) {
                connection.setRequestProperty(entry.getKey(), entry.getValue());
            }

            if (username != null && password != null) {
                // Step 6: Add Basic Authentication
                String auth = username + ":" + password;
                String encodedAuth = Base64.getEncoder().encodeToString(auth.getBytes());
                String authHeader = "Basic " + encodedAuth;
                connection.setRequestProperty("Authorization", authHeader);
            }

            if (requestPayload != null) {
                connection.setDoOutput(true);
                connection.getOutputStream().write(requestPayload.getBytes(StandardCharsets.UTF_8));
            }
            connection.connect();
        } catch (Exception e) {
            System.out.println("\nError occurred: Connection to URL: " + url + " using user/pass " + username + "/" + password + " failed - we expect Artemis-console to be available on HTTPS on port 8161 (hardcoded). Check user/pass + firewall + jolokia-access.xml to ensure access.");
            throw e;
        }

        return connection;
    }
}
